#include <qfile.h>
#include <qlabel.h>
#include <qsplitter.h>
#include <qtextbrowser.h>

#include <kdebug.h>
#include <klistview.h>
#include <klocale.h>
#include <kurl.h>

#include "xmldemo.h"
#include "xmldemo.moc"

static const QString DEMO_FILE = "contents.xml";

class DocNodeItem : public KListViewItem
{
public:
    DocNodeItem( KListView *parent, QDomElement tag )
	: KListViewItem( parent )
	{
	    init( tag );
	}

    DocNodeItem( KListViewItem *parent, QDomElement tag )
	: KListViewItem( parent )
	{
	    init( tag );
	}

    ~DocNodeItem() {}

    void init( QDomElement elem )
	{
	    id = elem.attribute( "docid" );
	    href = elem.attribute( "href" );

	    QDomNodeList textNodes = elem.elementsByTagName( "text" );
	    QDomElement textElem = textNodes.item(0).toElement();
	    title = ( textElem.isNull() ? "No Text!" : textElem.text() );

	    setText( 0, title );
	}

    QString href;
    QString title;
    QString id;
};

XMLDemo::XMLDemo()
    : QVBox( 0, "XMLDemo" ),
      doc(), toc(0)
{
    addHeading( i18n("XML Demo"), this );

    QSplitter *split = new QSplitter( this );
    split->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );

    toc = new KListView( split );
    toc->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Expanding ) );
    toc->setRootIsDecorated( true );
    toc->addColumn( "Title" );
    connect( toc, SIGNAL( selectionChanged(QListViewItem *) ),
	     this, SLOT( selected(QListViewItem *) ) );

    view = new QTextBrowser( split );
    view->setSizePolicy( QSizePolicy( QSizePolicy::Maximum, QSizePolicy::Expanding ) );

    kdDebug() << "loading file " << DEMO_FILE << endl;

    if (!load( DEMO_FILE )) {
	(void) new QLabel( QString("Error loading '%1'").arg( DEMO_FILE ), this );
	return;
    }
}

XMLDemo::~XMLDemo()
{
}

void XMLDemo::selected( QListViewItem *lvi )
{
    kdDebug() << "selection changed" << endl;
    DocNodeItem *i = static_cast<DocNodeItem *>( lvi );
    QFile f( i->href );

    if ( !f.open( IO_ReadOnly ) )
	return;

    QTextStream ts( &f );
    view->setText( ts.read() );
    f.close();
}

bool XMLDemo::load( const QString &filename )
{
    QFile f( filename );

    if ( !f.open( IO_ReadOnly ) )
	return false;

    if ( !doc.setContent( &f ) ) {
      f.close();
      return false;
    }

    f.close();

    updateContents();
    return true;
}

void XMLDemo::updateContents()
{
    toc->clear();
    processDocument( doc );
    toc->setSelected( toc->firstChild(), true );
}

//
// DOM Navigation
//

void XMLDemo::processDocument( QDomDocument &node )
{
    // Find root tag <toc>
    QDomNodeList docs = node.elementsByTagName( "toc" );
    QDomNode parent = docs.item( 0 );

    KListViewItem *rootItem = 0;
    QDomElement el = parent.toElement();
    processChildren( rootItem, el );
}

KListViewItem *XMLDemo::processDocTag( KListViewItem *parent, QDomElement &elem )
{
    KListViewItem *item;
    if ( parent )
	item = new DocNodeItem( parent, elem );
    else
	item = new DocNodeItem( toc, elem );

    processChildren( item, elem );

    item->setOpen( true );
    return item;
}

void XMLDemo::processChildren( KListViewItem *parent, QDomElement &elem )
{
    // For each child
    QDomNode ch = elem.firstChild();
    while ( !ch.isNull() ) {
	if ( !ch.isElement() )
	    break;
	QDomElement curr = ch.toElement();

	if ( curr.tagName() == "doc" )
	    processDocTag( parent, curr );

	ch = ch.nextSibling();
    }
}

//
// Utility Methods
//

QWidget *XMLDemo::addHeading( const QString &text, QWidget *w )
{
    QLabel *heading = new QLabel( text, w );
    heading->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed ) );
    heading->setMargin( 6 );

    heading->setAlignment( AlignCenter );
    QFont f = heading->font();
    f.setPointSize( 24 );
    f.setBold( true );
    heading->setFont( f );

    heading->setFrameStyle( QFrame::Panel|QFrame::Plain );
    heading->setLineWidth( 3 );

    return heading;
}

QWidget *XMLDemo::addSection( const QString &text, QWidget *w )
{
    QLabel *heading = new QLabel( text, w );
    heading->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed ) );

    heading->setAlignment( AlignVCenter|AlignLeft );
    QFont f = heading->font();
    f.setPointSize( 18 );
    f.setBold( true );
    heading->setFont( f );

    heading->setFrameStyle( QFrame::Panel|QFrame::Plain );
    heading->setLineWidth( 2 );

    return heading;
}

// Local Variables:
// c-basic-offset: 4
// End:
